"""
Unit type
"""
from enum import Enum
from typing import Optional, List

from pydantic import BaseModel


class RawUnit(BaseModel):
    unitId: str
    name: str


class RawRole(RawUnit):
    sequence: int


class RawTeam(RawUnit):
    sequence: int
    parentUnitId: str
    roles: List[RawRole]


class Mobile(BaseModel):
    areaCode: str
    number: str


class MemberTypeEnum(str, Enum):
    """
    Member's type.
    PrimaryAdmin: the owner of the space
    SubAdmin: the team,member,role manager
    Member: ordinary member
    """
    PrimaryAdmin = 'PrimaryAdmin'
    SubAdmin = 'SubAdmin'
    Member = 'Member'


class RawMember(RawUnit):
    avatar: str
    email: Optional[str]
    mobile: Mobile = Optional[Mobile]
    status: int
    type: MemberTypeEnum
    teams: List[RawTeam]
    roles: List[RawRole]


class ModifyMemberRequest(BaseModel):
    name: Optional[str]
    teams: Optional[List[str]]
    roles: Optional[List[str]]


class CreateRoleRequest(BaseModel):
    name: str
    sequence: Optional[int]


class ModifyRoleRequest(BaseModel):
    name: Optional[str]
    sequence: Optional[int]


class CreateTeamRequest(BaseModel):
    name: str
    sequence: Optional[int]
    # default "0" means root team
    parentUnitId: Optional[str]
    roles: Optional[List[str]]


class ModifyTeamRequest(BaseModel):
    name: Optional[str]
    sequence: Optional[int]
    parentUnitId: Optional[str]
    roles: Optional[List[str]]


class RoleUnit(BaseModel):
    teams: List[RawTeam]
    members: List[RawMember]


class PaginationUnit(BaseModel):
    pageSize: int
    pageNum: int
    total: int


class PaginationRole(PaginationUnit):
    roles: List[RawRole]


class PaginationTeam(PaginationUnit):
    teams: List[RawTeam]


class PaginationMember(PaginationUnit):
    members: List[RawMember]



"""
Unit of team instance.
"""
from vika.types.unit_model import CreateTeamRequest, RawTeam, ModifyTeamRequest, PaginationTeam, PaginationMember
from vika.unit import Unit


class Team(Unit):
    def create(self, data: CreateTeamRequest) -> RawTeam:
        """
        Add a team to the specified space.
        @param data: team information
        @return: RawTeam
        """
        r = self._space.apitable.request.post(self._team_endpoint,
                                              json=dict(data)).json()
        if r["success"]:
            return RawTeam(**r['data']['team'])
        else:
            raise Exception(r['message'])

    def update(self, unit_id: str, data: ModifyTeamRequest) -> RawTeam:
        """
        Update team's information roles information.
        @param data: team information
        @param unit_id: team's unit id
        @return: RawTeam
        """
        r = self._space.apitable.request.put(self._describe_team_endpoint(unit_id),
                                             json=dict(data)).json()
        if r["success"]:
            return RawTeam(**r['data']['team'])
        else:
            raise Exception(r["message"])

    def delete(self, unit_id: str) -> bool:
        """
        Remove a team from the specified space, notice: Only empty team can be deleted，
        @param unit_id: team's unit id
        @return: bool
        """
        r = self._space.apitable.request.delete(self._describe_team_endpoint(unit_id)).json()
        return r['success']

    def list_sub_teams(self, unit_id: str, page_num=1, page_size=100) -> PaginationTeam:
        """
        Paging to get the sub list of teams.
        @param unit_id: parent team’s unitId, root team id is "0"
        @param page_num: the page number of the pagination
        @param page_size: this parameter only accepts integers from 1-1000
        """
        r = self._space.apitable.request.get(self._list_sub_team_endpoint(unit_id),
                                             params={
                                                 'pageNum': page_num,
                                                 'pageSize': page_size
                                             }).json()
        if r["success"]:
            return PaginationTeam(**r['data'])
        else:
            raise Exception(r["message"])

    def list_members(self, unit_id: str, sensitive_data=False, page_num=1, page_size=100) -> PaginationMember:
        """
        List members under the team.
        @param sensitive_data: fill in “true“ to return member’s sensitive data, includes mobile number and email
        @param unit_id: current team’s unitId, root team id is "0"
        @param page_num: the page number of the pagination
        @param page_size: this parameter only accepts integers from 1-1000
        """
        r = self._space.apitable.request.get(self._list_team_member_endpoint(unit_id),
                                             params={
                                                 "sensitiveData": sensitive_data,
                                                 'pageNum': page_num,
                                                 'pageSize': page_size
                                             }).json()
        if r["success"]:
            return PaginationMember(**r['data'])
        else:
            raise Exception(r["message"])



import unittest
from unittest.mock import patch

from vika.space import Space


class TestApitable(unittest.TestCase):
    """
    base test class.
    """

    def setUp(self):
        """
        set up.
        """
        with patch('apitable.apitable') as mock_apitable:
            self._apitable = mock_apitable
            self._space = Space(self._apitable, 'test_space_id')



import unittest
from unittest import mock
from unittest.mock import MagicMock

from vika.types import CreateRoleRequest, ModifyRoleRequest, RawRole
from test.test_apitable import TestApitable


class TestUnit(TestApitable):

    def test_create_role_with_name_exist(self):
        # space = Space(mock_apitable, "mock_space")
        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_response.json.return_value = {
            "success": False,
            "message": 'A role with the same name exists',
            "code": 1509
        }
        self._apitable.request.post.return_value = mock_response
        self._space.role._role_endpoint = mock.Mock(return_vaule='/test_endpoint')
        with self.assertRaises(Exception) as context:
            self._space.role.create(CreateRoleRequest(name="test_role"))
            self.assertEquals(context.exception.__str__(), 'A role with the same name exists')

    def test_create_role_with_no_permission(self):
        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_response.json.return_value = {
            "success": False,
            "message": 'This API is only available for the primary admin and sub-admins who can manage roles',
            "code": 1508
        }
        self._apitable.request.post.return_value = mock_response
        self._space.role._role_endpoint = mock.Mock(return_vaule='/test_endpoint')
        with self.assertRaises(Exception) as ct:
            self._space.role.create(CreateRoleRequest(name="test_role"))
            self.assertEquals(ct.exception.__str__(),
                              'This API is only available for the primary admin and sub-admins who can manage roles')

    def test_create_role_success(self):
        mock_response = MagicMock()
        mock_response.status_code = 201
        mock_response.json.return_value = {
            "success": True,
            "message": 'SUCCESS',
            "code": 200,
            "data": {
                "role": {
                    "unitId": 'test_unit_id',
                    "name": 'test_role',
                    "sequence": 2000
                }
            }
        }
        self._apitable.request.post.return_value = mock_response
        self._space.role._role_endpoint = mock.Mock(return_vaule='/test_endpoint')
        role = self._space.role.create(CreateRoleRequest(name="test_role"))
        self.assertEqual(role, RawRole(**mock_response.json.return_value['data']['role']))

    def test_request_role_api_server_error(self):
        mock_response = MagicMock()
        mock_response.status_code = 500
        mock_response.json.return_value = {
            "success": False,
            "message": 'Server error',
            "code": 500
        }
        self._apitable.request.post.return_value = mock_response
        self._space.role._role_endpoint = mock.Mock(return_vaule='/test_endpoint')
        with self.assertRaises(Exception) as ct:
            self._space.role.create(CreateRoleRequest(name="test_role"))
            self.assertEquals(ct.exception.__str__(), 'Server error')

    def test_update_role_with_not_exist(self):
        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_response.json.return_value = {
            "success": False,
            "message": 'The unit specified by the unit ID does not exist',
            "code": 400
        }
        self._apitable.request.post.return_value = mock_response
        self._space.role._describe_role_endpoint = mock.Mock(return_vaule='/test_endpoint')
        with self.assertRaises(Exception) as ct:
            self._space.role.update('test_role', ModifyRoleRequest(name="test_role"))
            self.assertEquals(ct.exception.__str__(), 'The unit specified by the unit ID does not exist')


if __name__ == "__main__":
    unittest.main()
